-- #####################################################################################
-- ## Skript zum Workshop <SQLBattleShips>                                            ##     
-- #####################################################################################
-- ## Dieses Skript legt Datenbankobjekte fuer den Workshop <SQLBattleShips> an. Es   ##
-- ## wird durch weitere Skripte ergaenzt, die in ihrer durchnummerierten Reihenfolge ##
-- ## abzuarbeiten sind. Die Skripte passen zu den Ausfuehrungen aus der zugehoerigen ##
-- ## Powerpoint-Praesentation vom selben Autor.                                      ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Bitte dieses Skript niemals in einer Produktionsumgebung einsetzen, da          ##
-- ## Datenbankobjekte angelegt, ergaenzt, geaendert oder sogar geloescht werden!!!   ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Verwendung auf eigene Gefahr...                                                 ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Torsten Ahlemeyer, arelium GmbH        torsten.ahlemeyer@arelium.de             ##
-- #####################################################################################

-- auf die Workshops-Datenbank wechseln
USE [Vortrag_Kopfnuss]
GO

-- Gibt an, dass sich die Vergleichsoperatoren Gleich (=) und Ungleich (<>) bei Verwendung mit NULL-Werten in SQL Server 2019 (15.x) ISO-konform verhalten muessen.
-- ANSI NULLS ON ist neuer T-SQL Standard und wird in spaeteren Versionen festgeschrieben.
SET ANSI_NULLS ON
GO

-- Bewirkt, dass SQL Server die ISO-Regeln fuer Anfuehrungszeichen bei Bezeichnern und Literalzeichenfolgen befolgt.
SET QUOTED_IDENTIFIER ON
GO

--------------------------------------------------------------------------------------------------
-- Statistiken -----------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------

-- temporaere Tabelle anlegen, um sich die Startzeit zu merken
BEGIN TRY
	DROP TABLE #Start
END TRY
BEGIN CATCH
END CATCH

CREATE TABLE #Start (StartTime DATETIME)
INSERT INTO #Start (StartTime) VALUES (GETDATE())

-----------------------------
-- Aufraeumarbeiten ----------
-----------------------------
-- bestehendes Schemata loeschen, um sauber wieder aufzusetzen
DROP SCHEMA  IF EXISTS [BattleShips]
GO

-----------------------------
-- Aufbauarbeiten ----------
-----------------------------
-- Schemata fuer die Trennung der Spieler. So kann man ueber die Rechteverwaltung bei Bedarf jede Schummelform ausschlieen.
CREATE SCHEMA [BattleShips]
GO

-- ######################################################################################
-- ### Tabellen loeschen, um sie spaeter sauber wieder aufbauen zu koennen            ###
-- ######################################################################################

-- Tabellen im Schema [BattleShips]
-----------------------------------
DROP TABLE IF EXISTS [BattleShips].[ListeEigenesSpielbrett]
GO


-- ######################################################################################
-- ### Tabellen anlegen                                                               ###
-- ######################################################################################

-- Anlage der Tabelle [BattleShips].[ListeEigenesSpielbrett]. Sie enthaelt alle Felder des eigenen Spielbrettes und liegt als 
-- Liste mit 10x10 also 100 Eintraegen vor. Dabei sind die Spalte [Zielkoordinate] und die Kombination aus den Spalten 
-- [Reihe] und [Spalte] redundant und nur der einfacheren Abfrage wegen so angelegt worden.
-- ueber das Feld [Inhalt] wird nachgehalten, welche Schiffe wo stehen - hier ist das jeweilige Symbol des Bootes einzutragen.
-- Schiet der Gegner, wird die entsprechende Schussnummer im Feld [ZugnummerGegner] protokolliert.
----------------------------------
-- Diese Tabelle dient als Datenquelle fuer die Views zur Ansicht der Spielbretter
CREATE TABLE [BattleShips].[ListeEigenesSpielbrett](
	  [Zielkoordinate]				[VARCHAR](3)			NOT NULL					-- beinhaltet die komplette Zielkkordinate, also bspw. 'H6'
	, [Reihe]						[TINYINT]				NOT NULL					-- beinhaltet den Reihenanteil der Zielkoordinate, also bspw. '6'
	, [Spalte]						[CHAR](1)				NOT NULL					-- beinhaltet den Spaltenanteil der Zielkoordinate, also bspw. 'H'
	, [Inhalt]						[CHAR](1)				NULL						-- ist entweder leer (NULL) oder beinhaltet das Symbol fr ein Schiff, bspw. 'M'
	, [Anzeige]						[CHAR](1)				NULL						-- wie [Inhalt], verdeckt aber die Identitt des Schiffes durch den Platzhalter 'X' (bis das Schiff vollstndig versenkt wird)
	, [ZugnummerGegner]				[TINYINT]				NULL						-- gibt an, in wievielten Zug der Gegner auf dieses Feld geschoosen hat. NULL bedeutet unbeschossen
 CONSTRAINT [PK_ListeEigenesSpielbrett] PRIMARY KEY CLUSTERED 
(
	[Zielkoordinate] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON, OPTIMIZE_FOR_SEQUENTIAL_KEY = OFF) ON [PRIMARY]
) ON [PRIMARY]
GO

-- Ein CONSTRAINT ist eine Einschrnkung bzgl. der erlaubten Feldinhalte. Hier wird das Feld [Zielkoordinate] zum Primaerschluessel gemacht.
-- Diese drfen nicht NULL werden und sind stets eindeutig. Die Tabelle wird stets nach diesem Schlssel sortiert abgelegt.




-- ######################################################################################
-- ### Tabellen loeschen, um sie spaeter sauber wieder aufbauen zu koennen            ###
-- ######################################################################################

-- Tabellen im Schema [BattleShips]
-----------------------------------
DROP TABLE IF EXISTS [BattleShips].[ListeEigenesSchussfeld]
GO


-- ######################################################################################
-- ### Tabellen anlegen                                                               ###
-- ######################################################################################

-- Anlage der Tabelle [BattleShips].[ListeEigenesSchussfeld]. Sie enthaelt alle Felder des fremden Spielbrettes und liegt als 
-- Liste mit 10x10 also 100 Eintraegen vor. Dabei sind die Spalte [Zielkoordinate] und die Kombination aus den Spalten 
-- [Reihe] und [Spalte] redundant und nur der einfacheren Abfrage wegen so angelegt worden.
-- Ueber das Feld [Inhalt] wird nachgehalten, welche Schiffe wo stehen - hier ist das jeweilige Symbol des Bootes einzutragen.
-- Die Spalte [Anzeige] hingegen zeigt, welches Symbol in der grafischen Ansicht zu sehen sein wird - Schiffe bleiben 
-- "anonym", bis sie vollstndig versenkt wurden...
-- Schiet Spieler1 (blau), wird die entsprechende Schussnummer ueber das Feld [EigeneZugnummer] protokolliert.

-- Dieser Code ist ein Beispiel fr einen Defaultwert!
----------------------------------
-- Diese Tabelle dient als Datenquelle fuer die Views zur Ansicht der Spielbretter
CREATE TABLE [BattleShips].[ListeEigenesSchussfeld](
	  [Zielkoordinate]				[VARCHAR](3)			NOT NULL
	, [Reihe]						[TINYINT]				NOT NULL
	, [Spalte]						[CHAR](1)				NOT NULL
	, [Inhalt]						[CHAR](1)				NULL
	, [Anzeige]						[CHAR](1)				NULL
	, [EigeneZugnummer]				[TINYINT]				NULL				-- nur gueltige Schuesse
	, [GesamtSchusszahl]			[INTEGER]				NULL,				-- inkl. Fehlversuche
 CONSTRAINT [PK_ListeEigenesSchussfeld] PRIMARY KEY CLUSTERED 
(
	[Zielkoordinate] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON, OPTIMIZE_FOR_SEQUENTIAL_KEY = OFF) ON [PRIMARY]
) ON [PRIMARY]
GO

ALTER TABLE [BattleShips].[ListeEigenesSchussfeld]
  ADD CONSTRAINT [conAnzeigeNichtNull]
  DEFAULT ' ' FOR [Anzeige];
GO




------------------------------------------------------------------------------------------------------------------------------------------------------
-- Statistiken ---------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------
DECLARE @StartTime	DATETIME		= (SELECT StartTime FROM #Start)
DECLARE @Ende		VARCHAR(25)		= CONVERT(VARCHAR(25), GETDATE(), 104) + '   ' +CONVERT(VARCHAR(25), GETDATE(), 114)
DECLARE @Zeit		VARCHAR(500)	= CAST(DATEDIFF(SS, @StartTime, GETDATE()) AS VARCHAR(10)) + ',' + CAST(DATEPART(MS, GETDATE() - @StartTime) AS VARCHAR(10)) + ' sek.'
DECLARE @Skript		VARCHAR(100)	= '030 - Tabelle [ListeEigenesSpielbrett].sql'
PRINT ' '
PRINT 'Skript     :   ' + @Skript
PRINT 'Ende       :   ' + @Ende
PRINT 'Zeit       :   ' + @Zeit
SELECT @Skript AS Skript, @Ende AS Ende, @Zeit AS Zeit
GO
