-- #####################################################################################
-- ## Skript zum Workshop <SQLBattleShips>                                            ##     
-- #####################################################################################
-- ## Dieses Skript legt Datenbankobjekte fuer den Workshop <SQLBattleShips> an. Es   ##
-- ## wird durch weitere Skripte ergaenzt, die in ihrer durchnummerierten Reihenfolge ##
-- ## abzuarbeiten sind. Die Skripte passen zu den Ausfuehrungen aus der zugehoerigen ##
-- ## Powerpoint-Praesentation vom selben Autor.                                      ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Bitte dieses Skript niemals in einer Produktionsumgebung einsetzen, da          ##
-- ## Datenbankobjekte angelegt, ergaenzt, geaendert oder sogar geloescht werden!!!   ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Verwendung auf eigene Gefahr...                                                 ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Torsten Ahlemeyer, arelium GmbH        torsten.ahlemeyer@arelium.de             ##
-- #####################################################################################

-- auf die Workshops-Datenbank wechseln
USE [Vortrag_Kopfnuss]
GO

-- Gibt an, dass sich die Vergleichsoperatoren Gleich (=) und Ungleich (<>) bei Verwendung mit NULL-Werten in SQL Server 2019 (15.x) ISO-konform verhalten muessen.
-- ANSI NULLS ON ist neuer T-SQL Standard und wird in spaeteren Versionen festgeschrieben.
SET ANSI_NULLS ON
GO

-- Bewirkt, dass SQL Server die ISO-Regeln fuer Anfuehrungszeichen bei Bezeichnern und Literalzeichenfolgen befolgt.
SET QUOTED_IDENTIFIER ON
GO






--------------------------------------------------------------------------------------------------
-- Statistiken -----------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------

-- temporaere Tabelle anlegen, um sich die Startzeit zu merken
BEGIN TRY
	DROP TABLE #Start
END TRY
BEGIN CATCH
END CATCH

CREATE TABLE #Start (StartTime DATETIME)
INSERT INTO #Start (StartTime) VALUES (GETDATE())













-- ######################################################################################
-- ### Sichten loeschen, um sie spaeter sauber wieder aufbauen zu koennen             ###
-- ######################################################################################

DROP VIEW IF EXISTS [BattleShips].[vMatrixEigenesSpielbrett]
GO
DROP VIEW IF EXISTS [BattleShips].[vMatrixEigenesSchussfeld]
GO
DROP VIEW IF EXISTS [BattleShips].[vMatrixEigenesSpielbrettUndSchussfeld]
GO





-- ######################################################################################
-- ### Sichten erstellen                                                              ###
-- ######################################################################################

-- Diese View wandelt die Liste, in der die Schiffe und die Schuesse des Gegners gespeichert werden, in 
-- eine Matrixdarstellung um, um sie dann zur Grafikausgabe verwenden zu koennen. Man "sieht" also 
-- wo die eigenen Schiffe stehen und wohin der Gegner schnon geschossen hat. Um die Darstellung 
-- zu erreichen wird die Liste aus der Quelltabelle pivotisiert. Dies ist wesentlich eleganter und 
-- effektiver, als durch die Liste zu loopen...
CREATE OR ALTER VIEW [BattleShips].[vMatrixEigenesSpielbrett]
AS

	SELECT [Reihe], [A], [B], [C], [D], [E], [F], [G], [H], [I], [J]    
	FROM  
	(SELECT [Spalte], [Reihe], [Anzeige]
		FROM [BattleShips].[ListeEigenesSpielbrett]) AS SourceTable  
	PIVOT  
	(  
	MAX([Anzeige])  
	FOR [Spalte] IN ([A], [B], [C], [D], [E], [F], [G], [H], [I], [J])  
	) AS PivotTable;  

GO

-- Diese View wandelt die Liste, in der die eigenen Schuesse gespeichert werden, in 
-- eine Matrixdarstellung um, um sie dann zur Grafikausgabe verwenden zu koennen. Man "sieht" also 
-- wohin man schnon geschossen hat und welche Schiffe des Gegners dabei getroffen wurden. Um die 
-- Darstellung zu erreichen wird die Liset aus der Quelltabelle pivotisiert. Dies ist wesentlich 
-- eleganter und effektiver, als durch die Liset zu loopen...
CREATE OR ALTER VIEW [BattleShips].[vMatrixEigenesSchussfeld]
AS
	SELECT [Reihe], [A], [B], [C], [D], [E], [F], [G], [H], [I], [J]    
	FROM  
	(SELECT [Spalte], [Reihe], [Anzeige]
		FROM [BattleShips].[ListeEigenesSchussfeld]) AS SourceTable  
	PIVOT  
	(  
	MAX([Anzeige])  
	FOR [Spalte] IN ([A], [B], [C], [D], [E], [F], [G], [H], [I], [J])  
	) AS PivotTable;  
GO






-- Diese View listet alle konfigurierten und noch aktiven (=nicht abgeschossenen) Schiffe auf
CREATE OR ALTER VIEW [BattleShips].[vEigeneAktuelleZiele]
AS
 	SELECT 
		[Inhalt]					AS [Inhalt]
		, COUNT([Inhalt])			AS [Laenge]
	FROM [BattleShips].[ListeEigenesSpielbrett]
	WHERE 1 = 1
		AND [Inhalt] <> ' '
		AND [Inhalt] IN 
			(
				SELECT DISTINCT
					[Inhalt]
				FROM [BattleShips].[ListeEigenesSpielbrett]
				WHERE 1 = 1
					AND [Inhalt] <> ' '
					AND [Inhalt] = [Anzeige]
			)
	GROUP BY [Inhalt]
GO

-- Diese View listet alle konfigurierten und noch aktiven (=nicht abgeschossenen) Schiffe auf
CREATE OR ALTER VIEW [BattleShips].[vFremdeAktuelleZiele]
AS
 	SELECT 
		[Inhalt]					AS [Inhalt]
		, COUNT([Inhalt])			AS [Laenge]
	FROM [BattleShips].[ListeEigenesSchussfeld]
	WHERE 1 = 1
		AND [Inhalt] <> ' '
		AND [Inhalt] NOT IN 
			(
				SELECT DISTINCT
					[Inhalt]
				FROM [BattleShips].[ListeEigenesSchussfeld]
				WHERE 1 = 1
					AND [Inhalt] <> ' '
					AND [Inhalt] = [Anzeige]
			)
	GROUP BY [Inhalt]
GO



-- aus der View [vAktuelleEigeneZiele] bauen wir eine grafische Anzeige
CREATE OR ALTER VIEW [BattleShips].[vEigeneRestschiffsanzeige]
AS
	SELECT 
		  ROW_NUMBER() OVER(ORDER BY [Laenge] ASC)		AS [Reihe]
		, [MyCommaSeparatedList]						AS [Bootstyp]
		, REPLICATE ( '@' ,[Laenge] )					AS [Laenge]
	FROM
	(
 		SELECT [Laenge], LEFT(MyCommaSeparatedList
			, LEN(MyCommaSeparatedList)-1) AS MyCommaSeparatedList
		FROM [BattleShips].[vEigeneAktuelleZiele] AS [c]

		CROSS APPLY

		(SELECT 
			CAST([Inhalt] AS NVARCHAR(10)) + ','
		FROM [BattleShips].[vEigeneAktuelleZiele] AS [o]
		WHERE 1=1
			AND [o].[Laenge] = c.[Laenge]
		FOR XML PATH('')) AS x(MyCommaSeparatedList)
	) dummy
 
	GROUP BY [Laenge], MyCommaSeparatedList
GO


-- aus der View [vAktuelleEigeneZiele] bauen wir eine grafische Anzeige
CREATE OR ALTER VIEW [BattleShips].[vFremdeRestschiffsanzeige]
AS
	SELECT 
		  ROW_NUMBER() OVER(ORDER BY [Laenge] ASC)		AS [Reihe]
		, [MyCommaSeparatedList]						AS [Bootstyp]
		, REPLICATE ( '@' ,[Laenge] )					AS [Laenge]
	FROM
	(
 		SELECT [Laenge], LEFT(MyCommaSeparatedList
			, LEN(MyCommaSeparatedList)-1) AS MyCommaSeparatedList
		FROM [BattleShips].[vFremdeAktuelleZiele] AS [c]

		CROSS APPLY

		(SELECT 
			CAST([Inhalt] AS NVARCHAR(10)) + ','
		FROM [BattleShips].[vFremdeAktuelleZiele] AS [o]
		WHERE 1=1
			AND [o].[Laenge] = c.[Laenge]
		FOR XML PATH('')) AS x(MyCommaSeparatedList)
	) dummy
 
	GROUP BY [Laenge], MyCommaSeparatedList
GO











-- Diese View stellt das eigene Spielbrett (auf dem die eigenen Schiffe stehen) und das Schussfeld 
-- (Anzeige, wohin man schon geschossen hat und was getroffen wurde) platzsparend nebeneinander.
-- Die beiden Matrixen werden beschriftet - links das eigene Spielbrett, rechts die Schussanzeige
CREATE OR ALTER VIEW [BattleShips].[vMatrixEigenesSpielbrettUndSchussfeld]
AS

	SELECT
		  ISNULL([RSAB].[Bootstyp], ' ')	AS [.Bootstyp]
		, ISNULL([RSAB].[Laenge], ' ')		AS [.Laenge]
		, '       '							AS [...]
		, [SB].[Reihe] AS [.#]
		, [SB].[A] AS [.A], [SB].[B] AS [.B], [SB].[C] AS [.C], [SB].[D] AS [.D], [SB].[E] AS [.E]
		, [SB].[F] AS [.F], [SB].[G] AS [.G], [SB].[H] AS [.H], [SB].[I] AS [.I], [SB].[J] AS [.J]
		, CASE ROW_NUMBER() OVER(ORDER BY [SB].[Reihe] ASC) 
				WHEN 1 THEN '     Spieler1            '  
				WHEN 6 THEN '<-- meine Schiffe           '  
				WHEN 8 THEN '                          meine Schuesse -->'  
				ELSE ' '
				END AS [,,]
		, [SF].[Reihe] AS [,#]
		, [SF].[A] AS [,A], [SF].[B] AS [,B], [SF].[C] AS [,C], [SF].[D] AS [,D], [SF].[E] AS [,E]
		, [SF].[F] AS [,F], [SF].[G] AS [,G], [SF].[H] AS [,H], [SF].[I] AS [,I], [SF].[J] AS [,J]
		, '       '								AS [,,,]
		, ISNULL([RSAR].[Bootstyp], ' ')			AS [,Bootstyp]
		, ISNULL([RSAR].[Laenge], ' ')			AS [,Laenge]
	FROM [BattleShips].[vMatrixEigenesSpielbrett] AS [SB]
	INNER JOIN [BattleShips].[vMatrixEigenesSchussfeld]	AS [SF]		ON [SB].[Reihe]		= [SF].[Reihe]
	LEFT JOIN [BattleShips].[vEigeneRestschiffsanzeige]		AS [RSAB]	ON [RSAB].[Reihe]	= [SB].[Reihe]
	LEFT JOIN [BattleShips].[vFremdeRestschiffsanzeige]		AS [RSAR]	ON [RSAR].[Reihe]	= [SB].[Reihe]
GO

------------------------------------------------------------------------------------------------------------------------------------------------------
-- Statistiken ---------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------
DECLARE @StartTime	DATETIME		= (SELECT StartTime FROM #Start)
DECLARE @Ende		VARCHAR(25)		= CONVERT(VARCHAR(25), GETDATE(), 104) + '   ' +CONVERT(VARCHAR(25), GETDATE(), 114)
DECLARE @Zeit		VARCHAR(500)	= CAST(DATEDIFF(SS, @StartTime, GETDATE()) AS VARCHAR(10)) + ',' + CAST(DATEPART(MS, GETDATE() - @StartTime) AS VARCHAR(10)) + ' sek.'
DECLARE @Skript		VARCHAR(100)	= '034 - BattleShips - Views des Amaturenbrettes.sql'
PRINT ' '
PRINT 'Skript     :   ' + @Skript
PRINT 'Ende       :   ' + @Ende
PRINT 'Zeit       :   ' + @Zeit
SELECT @Skript AS Skript, @Ende AS Ende, @Zeit AS Zeit
GO
