-- #####################################################################################
-- ## Skript zum Vortrag <die schwierigen Kopfnsse mit T-SQL gelst>                 ##     
-- #####################################################################################
-- ## Dieses Skript legt die Tabellen fr die Aufgabe "RubiksCube" an. Es wird durch  ##
-- ## weitere Skripte ergnzt, die in ihrer durchnummerierten Reihenfolge abzu-       ##
-- ## arbeiten sind. Die Skripte passen zu den Ausfhrungen aus der zugehrigen       ##
-- ## Powerpoint-Prsentation vom selben Autor.                                       ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Bitte dieses Skript niemals in einer Produktionsumgebung einsetzen, da          ##
-- ## Datenbankobjekte angelegt, ergnzt, gendert oder sogar gelscht werden!!!      ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Verwendung auf eigene Gefahr...                                                 ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Torsten Ahlemeyer, arelium GmbH        torsten.ahlemeyer@arelium.de             ##
-- #####################################################################################

-- auf die Projektdatenbank wechseln
USE [Workshop_Kopfnuesse]
GO

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO





-- -----------------------------------------------------------------------------------
-- Diese Funktion erstellt eine Tabelle mit allen denkbaren Zugkombinationen - also --
-- alle Start- und Zielpositionen und die jeweilige Bewegung, die die Startposition --
-- in die Zielposition berfhrt.                                                   --
-- -----------------------------------------------------------------------------------
-- Dazu startet man bei einem gelsten Wrfel (ID 'WWWWGGRRBBOOGGRRBBOOYYYY') und   --
-- fhrt von dieser Position aus jede erlaubte Bewegung, was zu mehreren            --
-- Zielpositionen fhrt. Jede dieser Zugkombinationen wird in der Tabelle           --
-- [RubiksCube].[ZugKombinationen] notiert - soweit sie dort noch nicht stehen.     --
-- -----------------------------------------------------------------------------------
CREATE OR ALTER PROCEDURE [RubiksCube].[fncZugKombinationenErfassen] 
(
	  @Eingangsposition		VARCHAR(24)
	, @AbbruchGroesse		INTEGER
)
AS
BEGIN
    DECLARE @NeueEingangsposition	AS VARCHAR(24)
    DECLARE @AlteEingangsposition	AS VARCHAR(24)
    DECLARE @AlteZielposition		AS VARCHAR(24)
	DECLARE @Ergebnis				AS VARCHAR(24)
	DECLARE @Zielposition			AS VARCHAR(24)
	DECLARE @TempPosition			AS VARCHAR(24)
	DECLARE @Rueckgabewert			AS VARCHAR(24)
	DECLARE @RETURN_VALUE			AS INTEGER

	TRUNCATE TABLE [RubiksCube].[ZugKombinationen]

	-- ---------------------------------------------------------
	-- Erstdrehung obere Ebene ---------------------------------
	-- ---------------------------------------------------------
	SET @TempPosition = [RubiksCube].[fncDreheObenLinks](@Eingangsposition)
    INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
    VALUES  (@Eingangsposition, 'DreheObenLinks', @TempPosition, 'FALSE')

	SET @TempPosition = [RubiksCube].[fncDreheObenRechts](@Eingangsposition)
    INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
    VALUES  (@Eingangsposition, 'DreheObenRechts', @TempPosition, 'FALSE')

	-- ---------------------------------------------------------
	-- Erstdrehung untere Ebene ---------------------------------
	-- ---------------------------------------------------------
	SET @TempPosition = [RubiksCube].[fncDreheUntenLinks](@Eingangsposition)
    INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
    VALUES  (@Eingangsposition, 'DreheUntenLinks', @TempPosition, 'FALSE')

	SET @TempPosition = [RubiksCube].[fncDreheUntenRechts](@Eingangsposition)
    INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
    VALUES  (@Eingangsposition, 'DreheUntenRechts', @TempPosition, 'FALSE')

	-- ---------------------------------------------------------
	-- Erstdrehung rechte Ebene ---------------------------------
	-- ---------------------------------------------------------
	SET @TempPosition = [RubiksCube].[fncDreheRechtsHoch](@Eingangsposition)
    INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
    VALUES  (@Eingangsposition, 'DreheRechtsHoch', @TempPosition, 'FALSE')

	SET @TempPosition = [RubiksCube].[fncDreheRechtsRunter](@Eingangsposition)
    INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
    VALUES  (@Eingangsposition, 'DreheRechtsRunter', @TempPosition, 'FALSE')

	-- ---------------------------------------------------------
	-- Erstdrehung linke Ebene ---------------------------------
	-- ---------------------------------------------------------
	SET @TempPosition = [RubiksCube].[fncDreheLinksHoch](@Eingangsposition)
    INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
    VALUES  (@Eingangsposition, 'DreheLinksHoch', @TempPosition, 'FALSE')

	SET @TempPosition = [RubiksCube].[fncDreheLinksRunter](@Eingangsposition)
    INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
    VALUES  (@Eingangsposition, 'DreheLinksRunter', @TempPosition, 'FALSE')

	-- ---------------------------------------------------------
	-- Erstdrehung vordere Ebene -------------------------------
	-- ---------------------------------------------------------
	SET @TempPosition = [RubiksCube].[fncDreheVorneLinks](@Eingangsposition)
    INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
    VALUES  (@Eingangsposition, 'DreheVorneLinks', @TempPosition, 'FALSE')

	SET @TempPosition = [RubiksCube].[fncDreheVorneRechts](@Eingangsposition)
    INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
    VALUES  (@Eingangsposition, 'DreheVorneRechts', @TempPosition, 'FALSE')

	-- ---------------------------------------------------------
	-- Erstdrehung hintere Ebene -------------------------------
	-- ---------------------------------------------------------
	SET @TempPosition = [RubiksCube].[fncDreheHintenLinks](@Eingangsposition)
    INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
    VALUES  (@Eingangsposition, 'DreheHintenLinks', @TempPosition, 'FALSE')

	SET @TempPosition = [RubiksCube].[fncDreheHintenRechts](@Eingangsposition)
    INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
    VALUES  (@Eingangsposition, 'DreheHintenRechts', @TempPosition, 'FALSE')



            

    WHILE 1 = 1
        AND (SELECT COUNT(*) FROM [RubiksCube].[ZugKombinationen]) < @AbbruchGroesse
        AND (SELECT COUNT(*) FROM [RubiksCube].[ZugKombinationen] WHERE [istabgearbeitet] = 'FALSE') > 0
    BEGIN
        DELETE FROM [RubiksCube].[Bewegungssatz]

        INSERT INTO [RubiksCube].[Bewegungssatz] 
            SELECT TOP 1 
				  [Ausgangsposition]
				, [Bewegung]
				, [Zielposition]
				, [istabgearbeitet]
            FROM [RubiksCube].[ZugKombinationen] WHERE [istabgearbeitet] = 'FALSE'

        SET @NeueEingangsposition	= (SELECT TOP 1 [Zielposition]		FROM [RubiksCube].[Bewegungssatz])
        SET @AlteEingangsposition	= (SELECT TOP 1 [Ausgangsposition]	FROM [RubiksCube].[Bewegungssatz])
        SET @AlteZielposition		= (SELECT TOP 1 [Zielposition]		FROM [RubiksCube].[Bewegungssatz])

        UPDATE [RubiksCube].[ZugKombinationen]
        SET [istabgearbeitet] = 'TRUE'
        WHERE 1 = 1
            AND [Ausgangsposition]	= @AlteEingangsposition   
            AND [Zielposition]		= @AlteZielposition

		-- ---------------------------------------------------------
		-- Drehung obere Ebene -------------------------------------
		-- ---------------------------------------------------------
		SET @TempPosition = [RubiksCube].[fncDreheObenLinks](@NeueEingangsposition)
        IF (SELECT COUNT(*) FROM [RubiksCube].[ZugKombinationen] WHERE [Ausgangsposition] = @NeueEingangsposition AND [Zielposition] = @TempPosition) = 0
            INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
            VALUES  (@NeueEingangsposition, 'DreheObenLinks', @TempPosition, 'FALSE')
            
		SET @TempPosition = [RubiksCube].[fncDreheObenRechts](@NeueEingangsposition)
        IF (SELECT COUNT(*) FROM [RubiksCube].[ZugKombinationen] WHERE [Ausgangsposition] = @NeueEingangsposition AND [Zielposition] = @TempPosition) = 0
            INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
            VALUES  (@NeueEingangsposition, 'DreheObenRechts', @TempPosition, 'FALSE')
            
		-- ---------------------------------------------------------
		-- Drehung untere Ebene ------------------------------------
		-- ---------------------------------------------------------
		SET @TempPosition = [RubiksCube].[fncDreheUntenLinks](@NeueEingangsposition)
        IF (SELECT COUNT(*) FROM [RubiksCube].[ZugKombinationen] WHERE [Ausgangsposition] = @NeueEingangsposition AND [Zielposition] = @TempPosition) = 0
            INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
            VALUES  (@NeueEingangsposition, 'DreheUntenLinks', @TempPosition, 'FALSE')
            
		SET @TempPosition = [RubiksCube].[fncDreheUntenRechts](@NeueEingangsposition)
        IF (SELECT COUNT(*) FROM [RubiksCube].[ZugKombinationen] WHERE [Ausgangsposition] = @NeueEingangsposition AND [Zielposition] = @TempPosition) = 0
            INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
            VALUES  (@NeueEingangsposition, 'DreheUntenRechts', @TempPosition, 'FALSE')
            
		-- ---------------------------------------------------------
		-- Drehung linke Ebene ------------------------------------
		-- ---------------------------------------------------------
            
		SET @TempPosition = [RubiksCube].[fncDreheLinksHoch](@NeueEingangsposition)
        IF (SELECT COUNT(*) FROM [RubiksCube].[ZugKombinationen] WHERE [Ausgangsposition] = @NeueEingangsposition AND [Zielposition] = @TempPosition) = 0
            INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
            VALUES  (@NeueEingangsposition, 'DreheLinksHoch', @TempPosition, 'FALSE')
            
		SET @TempPosition = [RubiksCube].[fncDreheLinksRunter](@NeueEingangsposition)
        IF (SELECT COUNT(*) FROM [RubiksCube].[ZugKombinationen] WHERE [Ausgangsposition] = @NeueEingangsposition AND [Zielposition] = @TempPosition) = 0
            INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
            VALUES  (@NeueEingangsposition, 'DreheLinksRunter', @TempPosition, 'FALSE')
            
		-- ---------------------------------------------------------
		-- Drehung rechte Ebene ------------------------------------
		-- ---------------------------------------------------------
            
		SET @TempPosition = [RubiksCube].[fncDreheRechtsHoch](@NeueEingangsposition)
        IF (SELECT COUNT(*) FROM [RubiksCube].[ZugKombinationen] WHERE [Ausgangsposition] = @NeueEingangsposition AND [Zielposition] = @TempPosition) = 0
            INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
            VALUES  (@NeueEingangsposition, 'DreheRechtsHoch', @TempPosition, 'FALSE')
            
		SET @TempPosition = [RubiksCube].[fncDreheRechtsRunter](@NeueEingangsposition)
        IF (SELECT COUNT(*) FROM [RubiksCube].[ZugKombinationen] WHERE [Ausgangsposition] = @NeueEingangsposition AND [Zielposition] = @TempPosition) = 0
            INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
            VALUES  (@NeueEingangsposition, 'DreheRechtsRunter', @TempPosition, 'FALSE')
            
		-- ---------------------------------------------------------
		-- Drehung vordere Ebene -----------------------------------
		-- ---------------------------------------------------------
            
		SET @TempPosition = [RubiksCube].[fncDreheVorneLinks](@NeueEingangsposition)
        IF (SELECT COUNT(*) FROM [RubiksCube].[ZugKombinationen] WHERE [Ausgangsposition] = @NeueEingangsposition AND [Zielposition] = @TempPosition) = 0
            INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
            VALUES  (@NeueEingangsposition, 'DreheVorneLinks', @TempPosition, 'FALSE')
            
		SET @TempPosition = [RubiksCube].[fncDreheVorneRechts](@NeueEingangsposition)
        IF (SELECT COUNT(*) FROM [RubiksCube].[ZugKombinationen] WHERE [Ausgangsposition] = @NeueEingangsposition AND [Zielposition] = @TempPosition) = 0
            INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
            VALUES  (@NeueEingangsposition, 'DreheVorneRechts', @TempPosition, 'FALSE')
            
		-- ---------------------------------------------------------
		-- Drehung hintere Ebene -----------------------------------
		-- ---------------------------------------------------------
            
		SET @TempPosition = [RubiksCube].[fncDreheHintenLinks](@NeueEingangsposition)
        IF (SELECT COUNT(*) FROM [RubiksCube].[ZugKombinationen] WHERE [Ausgangsposition] = @NeueEingangsposition AND [Zielposition] = @TempPosition) = 0
            INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
            VALUES  (@NeueEingangsposition, 'DreheHintenLinks', @TempPosition, 'FALSE')
            
		SET @TempPosition = [RubiksCube].[fncDreheHintenRechts](@NeueEingangsposition)
        IF (SELECT COUNT(*) FROM [RubiksCube].[ZugKombinationen] WHERE [Ausgangsposition] = @NeueEingangsposition AND [Zielposition] = @TempPosition) = 0
            INSERT INTO [RubiksCube].[ZugKombinationen] ([Ausgangsposition], [Bewegung], [Zielposition], [istabgearbeitet])
            VALUES  (@NeueEingangsposition, 'DreheHintenRechts', @TempPosition, 'FALSE')

    END
END
GO